'use strict';
var gutil = require('gulp-util');
var through = require('through2');
var cheerio = require('cheerio');
var hljs = require('highlight.js-mt');
var assign = require('lodash.assign');

var DEFAULTS = {
  cheerio: {
    decodeEntities: false
  }
};

module.exports = function (options) {
  var options = assign(DEFAULTS, options);
  var highlight = function (str, options) {
    var $ = cheerio.load(str, options.cheerio);
    $('code').each(function (index, code) {
      if (!$(code).hasClass('nohighlight')) {
        hljs.configure({
          classPrefix: ''
        })
        var html = $(code).html();
        var languageSubset = [];
        languageSubset.push($(code).attr('class'));
        $(code).html(hljs.highlightAuto(html, languageSubset).value);
      }
    });
    return $.html() || str;
  };
  return through.obj(function (file, enc, cb) {
    if (file.isNull()) {
      this.push(file);
      return cb();
    }

    if (file.isStream()) {
      this.emit('error', new gutil.PluginError('gulp-highlight', 'Streaming not supported'));
      return cb();
    }

    try {
      file.contents = new Buffer(highlight(file.contents.toString(), options), options.buffer);
    } catch (err) {
      this.emit('error', new gutil.PluginError('gulp-highlight', err));
    }

    this.push(file);
    cb();
  });
};
